function [DictionaryTopic, TopicDocument, log_lik, nr_topic_changed_list, DictionaryTopic_f, TopicDocument_f] ...
    = Gibbs_LDA(x, K, alpha, beta, max_iter, burnin_iter, sample_lag ...
    , plot_flag, word_reshape, print_flag, detail_print_flag)
%
% ************************** [Input argument] **************************
% x            : J by I matrix contains all observations (words) 
%                   J: Total number of documents
%                   I: Total number of words per document
% K            Number of topics per document 
%                   Topic is a multinomial distribution over words. 
% alpha        : Hyperparameter of G_0
% beta         : Hyperparameter of G_j
% max_iter     : Maximum number of iterations
% burnin_iter  : Burn-in iterations
% sample_lag   : Data collecting period 
% plot_flag    : 
% plot_phi     : 
%
%
% ************************** [Output argument] **************************
% DictionaryTopic: 
% TopicDocument
% logPw_z
% DictionaryTopic_f
% TopicDocument_f
%
% 
% ******* [Dimensions] ********
% J   : Number of documents
% I   : Number of words in a document
% V   : Number of vocabularies (total words) in a dictionary
% K   : Number of topics (distribution of words)
%
% ************** 1. First initialize Gibb sampler **************
[J, I] = size(x);
V = length(unique(x));
if print_flag
    fprintf('1. Initialize Gibbs sampler for Latent Dirichlet Allocation \n');
    fprintf(' Total %d documents, and %d words are in it. \n', J, I);
    fprintf(' Total %d unique words, meaning that the size of the dictionary is %d \n', V, V);
end

% The number of topics per each document 
NumTopicDocument = zeros(K, J) + alpha;
% The number of words per each topic
NumDictionaryTopic = zeros(V, K) + beta;
% The number of total words per each topic
NumTopic = sum(NumDictionaryTopic);
% Document <=> Observations
theta = zeros(J, I);
% Vocabulary <=> Topics
DictionaryTopic = zeros(V, K);
% Topics <=> Document
TopicDocument = zeros(K, J);

for j = 1:J % for each document
    for i = 1:I % for each word
        % Initialize each theta_{ji}. This indicates topic label.
        theta(j, i) = find(mnrnd(1, ones(1, K)/K)==1);
        % Initialize [Topic-Document]
        NumTopicDocument(theta(j,i), j) = NumTopicDocument(theta(j, i), j) + 1;
        % Initialize [Dictionary-Topic]
        NumDictionaryTopic(x(j,i), theta(j,i)) ...
            = NumDictionaryTopic(x(j,i), theta(j,i)) + 1;
        % Initialize [Topic]
        NumTopic(theta(j,i)) = NumTopic(theta(j,i)) + 1;
    end
end


% ************** 2. Gibbs sample for Latent Dirichlet Allocation **************
if print_flag
    fprintf('2. Start Gibbs sample for latent dirichlet allocation (LDA)\n');
end
save_DictionaryTopic = zeros(V, K);
save_TopicDocument = zeros(K, J);
save_nrSample = 0;
log_lik = zeros(1, max_iter);
nr_topic_changed_list = zeros(1, max_iter);
for iter = 1:max_iter
    % 0. Show current iteration 
    if (mod(iter, floor(max_iter/10)) == 0 || iter == 1) && print_flag
        fprintf('  [%d-%d] \n', iter, max_iter);
        [DictionaryTopic_t] = get_result(NumDictionaryTopic, NumTopicDocument, NumTopic, V, K, J);
    end
    
    % 1. Plot current status evey 50 iterations
    if (iter==1 || mod(iter, 50)==0) && plot_flag 
        fig_temp = figure(99); set(fig_temp, 'Position', [500 200 500 400]);
        colormap 'gray'
        for k=1:K
            subplot(2,K/2,k);
            imagesc(reshape(DictionaryTopic_t(:,k), word_reshape)); axis equal; axis tight; axis off;
            title(['Topic \theta_' num2str(k) ' Iter: ' num2str(iter)]);
            err = 1 - sum(DictionaryTopic_t(:,k));
            if abs(err) > 0.1, fprintf('Something wrong. \n'); pause; end;
        end
        drawnow;
        print('-dpng', sprintf('pics/lda_%03d.png', iter));
    end
    
    % 2. Update theta (topic) This part is everything
    nr_topic_changed = 0;
    for j = 1:J % for each document
        for i = 1:I % for each word
            
            % 1. Decrease three counts corresponding to current document and
            % word to exclude current observation
            NumTopicDocument(theta(j,i), j) ...
                = NumTopicDocument(theta(j,i), j) - 1;
            NumDictionaryTopic(x(j,i), theta(j,i)) ...
                = NumDictionaryTopic(x(j,i), theta(j,i)) - 1;
            NumTopic(theta(j,i)) = NumTopic(theta(j,i)) - 1;
            
            if detail_print_flag 
                fprintf('\n  [%d/%d] Document [%d/%d]-[%d/%d]th word [%d] has [%d]th topic. \n' ...
                    , iter, max_iter, j, J, i, I, x(j,i), theta(j,i));
            end
            
            % 2. Update the posterior distribution
            % The topic probability is proportional to 
            %   a. The likelihood of current word (How many cust.) given topic k
            %   b. The number of topic (tables) in current document (Resta.) 
            p_theta_ji = zeros(1, K);
            for k = 1:K
                curr_word = x(j,i);
                % a. Likelihood of current word in topic 
                %    (How many people are eating current dish k)
                nr_curr_word_in_topic_k = NumDictionaryTopic(curr_word, k);
                nr_total_word_in_topic_k = NumTopic(k);
                lik_curr_word_in_topic_k = nr_curr_word_in_topic_k/nr_total_word_in_topic_k;
                %  b. Number of current topic k in current document j
                %    (How many tables are serving current dish k)
                nr_curr_topic_k_in_doc_j = NumTopicDocument(k, j);
                % Posterior is proportional to above two:
                p_theta_ji(k) = lik_curr_word_in_topic_k ...
                    * nr_curr_topic_k_in_doc_j;
            end
            p_theta_ji = p_theta_ji / sum(p_theta_ji);
            if detail_print_flag
                fprintf('  The probability over topics is');
                fprintf(' [ ');
                fprintf('%.2f ', p_theta_ji);
                fprintf(']\n');
            end
            
            % 3. Draw topic for this word 
            prev_topic = theta(j,i);
            theta(j,i) = find(mnrnd(1, p_theta_ji)==1);
            if prev_topic ~= theta(j,i)
                nr_topic_changed = nr_topic_changed + 1;
            end
            if detail_print_flag
                fprintf('   => changed to [%d]th topic \n', theta(j,i));
            end
            
            % 4. Reincrease the count 
            NumTopicDocument(theta(j,i), j) ...
                = NumTopicDocument(theta(j,i), j) + 1;
            NumDictionaryTopic(x(j,i), theta(j,i)) ...
                = NumDictionaryTopic(x(j,i), theta(j,i)) + 1;
            NumTopic(theta(j,i)) = NumTopic(theta(j,i)) + 1;
        end
    end
    nr_topic_changed_list(iter) = nr_topic_changed;
    
    % 3. Compute Log Likelihood 
    
    
    % 4. Save samples for each sampling_lag (Theses values will be averaged to
    %   Our final result)
    if (mod(iter, sample_lag)==0 || iter == 1) && iter >= burnin_iter
        save_nrSample = save_nrSample + 1;
        % Dictionary-Topic
        for k = 1:K
            save_DictionaryTopic(:, k) = save_DictionaryTopic(:, k) ...
                + NumDictionaryTopic(:, k)/NumTopic(k);
        end
        % Topic-Document
        for j = 1:J
            save_TopicDocument(:, j) = save_TopicDocument(:, j) ...
                + NumTopicDocument(:, j)/sum(NumTopicDocument(:, j));
        end 
    end
end


% ************** 3. Compute DictionaryTopic and Dictionary Topic **************
% Compute average
DictionaryTopic = save_DictionaryTopic /save_nrSample;
TopicDocument = save_TopicDocument / save_nrSample;
% Compute last result 
DictionaryTopic_f = zeros(V, K);
TopicDocument_f = zeros(K, J);
for k = 1:K
    DictionaryTopic_f(:, k) = NumDictionaryTopic(:, k)/NumTopic(k);
end
for j = 1:J
    TopicDocument_f(:,j) = NumTopicDocument(:,j)/sum(NumTopicDocument(:,j));
end


function [DictionaryTopic, TopicDocument] ...
    = get_result(NumDictionaryTopic, NumTopicDocument, NumTopic, V, K, J)

DictionaryTopic = zeros(V, K);
TopicDocument = zeros(K, J);
for k = 1:K
    DictionaryTopic(:, k) = NumDictionaryTopic(:, k)/NumTopic(k);
end
for j = 1:J
    TopicDocument(:,j) = NumTopicDocument(:,j)/sum(NumTopicDocument(:,j));
end







