% Generate graphical example for topic modeling
%
% sungjoon.s.choi@gmail.com
% 

% plot_fig = 1;
% save_png = 0;
% show_text = 0;

% 1. First, we assume that we have 16 WORDs per documents
% Note that it is 'exchangable'.
if plot_fig
    fig_word = figure(1); set(fig_word, 'Position', [200 200 500 400]); colormap 'gray'
    imagesc(reshape(zeros(16,1), [4 4])); axis equal; axis tight;
    title('Bag of Words - Each sqaure represents a word', 'FontSize', 20);
    grid on;
    set(gca, 'xtick', [-0.5:1:4]);
    set(gca, 'ytick', [-0.5:1:4]);
    drawnow;
    if save_png
        print('-dpng', 'pics/words.png');
    end
end

% 2. Now, lets generate TOPICs
% Topics are multinomial parameters over words
% Here, we assume that each topic contains equal probability, 0.25, over
% four words. It would be something like
% topic_k = (0, 0, ..., 0.25, 0.25, 0.25, 0.25, 0, 0, ..., 0)
topics(:,1) = [1 0 0 0 1 0 0 0 1 0 0 0 1 0 0 0]/4;
topics(:,2) = [0 1 0 0 0 1 0 0 0 1 0 0 0 1 0 0]/4;
topics(:,3) = [0 0 1 0 0 0 1 0 0 0 1 0 0 0 1 0]/4;
topics(:,4) = [0 0 0 1 0 0 0 1 0 0 0 1 0 0 0 1]/4;
topics(:,5) = [1 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0]/4;
topics(:,6) = [0 0 0 0 1 1 1 1 0 0 0 0 0 0 0 0]/4;
topics(:,7) = [0 0 0 0 0 0 0 0 1 1 1 1 0 0 0 0]/4;
topics(:,8) = [0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 1]/4;
% show ground truth of topics (distribution over words)
if plot_fig
    fig_word = figure(2); set(fig_word, 'Position', [300 200 500 400]);
    colormap 'gray'
    for i=1:8
        subplot(2,4,i); imagesc(reshape(topics(:,i), [4 4])); axis equal; axis tight; % axis off;
        title(['Topic \theta_' num2str(i)])
    end;
    drawnow;
    if save_png
        print('-dpng', 'pics/topic_theta_ground_truth.png');
    end
end

% 3. Lets generate DOCUMENTs with given topics
alpha = 5;   
beta  = 1;
J     = 500; % number of document
K     = 8;   % number of topic
I     = 100; % number of words in each document
V     = 16;  % size of the vocabulary (#Total words)
DIC = 'abcdefghijklmnopqrstuwxyz';

theta = zeros(J, I); % Total corpus (Bag of Words)
x     = zeros(J, I); % Contains index of word
Gj    = zeros(J, K); 
word_hist = zeros(4, 4, J);


fprintf('Generating %d documents (bag of words) from topics. \n', J);
fprintf('Total size of dictionary (#words) is %d. \n', I);
fprintf('Each document contains %d topics and %d words. \n', K, I);
for j = 1:J
    % For each jth document, fill words.
    if show_text, fprintf('For %dth document \n', j); end
    
    % Sample from dirichlet distribution generates multinomial parameters.
    % This vector of size K works as a topic of the words in this documnet.
    Gj(j, :) = dirrnd(alpha*ones(1, K));
    % So, basically, Gj indicates distribution over topics which is a
    % dirichlet random variables. 
    if show_text, fprintf('  Currently drawn distribution over topics, G_%d, is\n ', j); disp(Gj(j, :)); end
    
    
    for i = 1:I
        % For each ith word in current jth document
        if show_text, fprintf('     For each %dth word in %dth document, \n', i, j); end
        
        % First, draw a topic from Gj
        % Note that a topic is a distribution over words!
        theta(j, i) = find(mnrnd(1, Gj(j, :))==1);
        
        % Second, draw a word from the topic
        word_dist = reshape(topics(:,theta(j,i)),[1 V]);
        x(j,i) = find(mnrnd(1, word_dist)==1);
        if show_text, fprintf('     Topic [%d] => Word [%d (%c)] is drawn. \n'...
            , theta(j, i), x(j, i), DIC(x(j, i))); end
    end
    
    % Make histogram of words of j-th documents 
    word_hist(:, :, j) = reshape(hist(x(j, :), 16), 4, 4);
    % Each number in a matrix word_hist(:, :, j) indicates the number of 
    % occurance of corresponding word in j-th document 
end

% 4. Visualize the corpus (bag of documents) only 12 of them
if plot_fig
    fig_word = figure(3); set(fig_word, 'Position', [400 200 700 600]);
    colormap 'winter'
    plotw = 3;
    ploth = 4;
    for i = 1:plotw*ploth
        subplot(plotw, ploth, i); 
        imagesc(word_hist(:,:,i)); axis equal; axis tight; %axis off;
        colorbar
        title(['Words of ' num2str(i) '-th doc']);
    end;
    drawnow;
    if save_png
        print('-dpng', 'pics/example_documents.png');
    end
end

